// Fichier Led.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                 Librairie de gestion des Leds		*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *  Les CAN doivent tre dsactive                     *
 ********************************************************/

#include <p18f452.h>									// Librairie du PIC utilis


#include "tourelle.h"									// Entte de la librairie 


// --------------------------------------------------------------------------
/* Initialise la tourelle
 *  Pas de paramtre
*/
void Init_Tourelle(void)
{
	TRIS_FOURCHE = 1;									// Configure le port de la fourche en entre
	TRIS_REFLEXE = 1;									// Configure le port du capteur en entre
	
	Front_Reflexe();							// Initialise les fonctions de dtection des fronts
	Front_Descendant_Fourche();			
}




// --------------------------------------------------------------------------
/* Retourne un si un front descendant est dtect sur la fourche
 *  Pas de paramtre
*/
int Front_Descendant_Fourche(void)
{
	static unsigned char Fourche_Old=1;					// Variable qui mmorise l'tat prcdent
	unsigned char Fourche=FOURCHE;						// Lit la valeur courante du capteur
	
	if (!Fourche && Fourche_Old)						// Si on dtecte un front descendant
	{						
		Fourche_Old=Fourche;							// On mmorise l'tat du capteur						
		return 1;										// On retourne vrai
	}
	Fourche_Old=Fourche;								// On mmorise l'tat du capteur						
	return 0;											// On retourne faux
}





// --------------------------------------------------------------------------
/* Retourne un si un front descendant est dtect sur le capteur reflexe
 *  Pas de paramtre
 * Si la fonction retourne 0, c'est que le capteur ne capte pas
 * Si la fonction retourne 1, c'est que le capteur capte
 * Si la fonction retourne 2, c'est que l'on vient de dtecter un front montant, (on vient de capter)
 * Si la fonction retourne 3, c'est que l'on vient de dtecter un front descendant, (on ne capte plus)
*/
int Front_Reflexe(void)
{
	static unsigned char Mesure_Old=1;					// Variable qui mmorise l'tat prcdent
	unsigned char Mesure=REFLEXE;						// Lit le capteur 
	unsigned char Ret=Mesure;							// Valeur de retour, l'tat courant du capteur par dfaut
	
	if (Mesure && !Mesure_Old)							// Si on dtecte un front montant sur le capteur						
		Ret=2;											// On retourne 2

	if (!Mesure && Mesure_Old)							// Si on dtecte un front descendant sur le capteur						
		Ret=3;											// On retourne 3

	Mesure_Old=Mesure;									// On mmorise l'tat du capteur						
	return Ret;											// On retourne faux
}




