// Fichier main.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : Guibert David                              		*       
 *           Cadic Matthieu										*
 *           Luzeau Yohan                                		*
 * .Descriptions :                                      		*
 *      - Fichier principal pour la gestion du rouleau droit   	*
 *                                                      		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *      - Gestion des ordre reus par interruption plutt		*
 *		  que par polling								   		*
 *      - Adresse de la carte droite 0x4E          				*
 * 		- Transmission IC sur interruption (priorit haute)	*
 * 		- Timer sur interruption (priorit basse)               *
 *		- La led rouge marque l'interruption timer				*
 *		- La led ambre marque une transmission IC				*
 *																*
 ****************************************************************/




// ------------ Directives de compilation
#pragma config WDT = OFF	// Pas de  watchdog
#pragma config OSC = HS		// Quartz HS pour un quartz 20MHz


// --------------------------------------------------------------------------
// Enttes ncessaires
#include <p18f252.h>		// Pour les registres du PIC18F252
#include <delays.h>    		// Pour les temporisations
#include <timers.h>			// Pour la base de temps de 10ms
#include <adc.h>			// Bibliothque des convertisseurs Analogiques Numriques
#include <stdio.h>
#include <math.h>

#include "Moteur.h"			// Librairie de gestion du moteur
#include "I2C_Slave.h"		// Librairie pour l'I2C
#include "led.h"			// Librairie pour les leds
#include "codeur.h"			// Librairie pour le codeur
#include "serial.h"
#include "Tourelle.h"
//#include "Triangulation.h"

// --------------------------------------------------------------------------
// Constantes

//#define		K			2.202609867
//#define		Alpha		0.93299

#define			AFFICHAGE_SERIE

#define			K				0.7
#define			Alpha			0.93299
#define 		COMMANDE_MAX	300				// Commande max applique sur les moteurs
// XM, 27/04
//#define			VIT_TOURELLE	55				// Vitesse de rotation de la tourelle
#define			VIT_TOURELLE	70				// Vitesse de rotation de la tourelle

// XM, 27/04, ajout de
#define			LMIN	30						// largeur minimum de la raie (robot adverse  3m)
#define			LMAX	300						// largeur maximum de la raie (robot adverse "contre" nous)

#define 		T_TIMER3 		40535		   	// 5 ms avant dbordement du Timer
												// Priode du quartz : 1/20e6 = 50ns
												// Priode  l'entre du prescaler : 50ns.4 = 200ns
												// Priode  la sortie du prescaler : 200ns.1 = 200ns
												// Nombre de priodes avant dbordement du timer : 10000 incrments
												// Priode entre deux interruptions : 25000*200ns = 5ms
												// Priode de 1ms -> 1KHz
												// 60536 pour 1ms
												// 55536 pour 2ms
												// 15535 pour 10ms





// --------------------------------------------------------------------------

// Prototype de la routine de service d'interuptions de priorit haute
void Interrupt_High_Priority(void);						// Utilise pour l'IC


// Prototype de la routine de service d'interuptions de priorit basse
void Interrupt_Low_Priority(void);						// Utilise pour le timer

unsigned char 	AngleRobotAdverse=0;					// Angle avec le robot adverse entre 0 et 255 (Partag avec l'interruption I2C)
unsigned char 	DistaRobotAdverse=0;					// "Distance" avec le robot adverse entre 0 et 255 (Partag avec l'interruption I2C)
unsigned char	Donnee_Recue=0;							// Donne reue par l'I2C

char			ConsigneTourelle=0;						// Consigne  appliquer sur le moteur (Partag avec la routine d'interruption pour l'asservissement)
int				PositionTourelle=0;						// Position absolue de la tourelle (Partag avec la routine d'interruption pour l'asservissement)


	
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Fonction Principale                          //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

void main (void)
{

	unsigned char FOURCHE_OLD=FOURCHE;					// Ancienne valeur de la fourche
	unsigned char REFLEXE_OLD=REFLEXE;					// Ancienne valeur du capteur reflexe
	unsigned char State=0;								// Etat courant de la machine  tat (0 car on attends toujours un front montant pour dmarrer)
	
	int Debut=-1,Fin=-1;								// Dbut et fin de la dtection d ela balise
	int DebutMax,FinMax;								// Dbut et fin de la meilleure trame connue
	int MeilleurPosition;								// Meilleure position connue de l'adversaire
	int LargeurRaieMax=-1;								// Meilleure largeure de raie connue
	int Largeur;										// Largeur de la raie courante



	// Configure le micro		
	OpenADC (ADC_0ANA_0REF,ADC_INT_OFF);				// Desactive les convertisseurs Analogiques/Numriques
	Moteur_Init();										// Initialise la commande du moteur
 	Led_Init();											// Configure les ports pour les leds
 	Codeur_Init();										// Initialise les codeurs
 	while (Codeur_Vitesse()!=0);						// Attends que la tourelle ne tourne pas
	Serial_Init();										// Initialise la liaison srie	
 	I2C_Init ();										// Initialise l'I2C
 	
	


	// Configuration du timer 3 pour la base de temps
	
	OpenTimer3(	TIMER_INT_ON &							// Autorise les interruption sur le timer 3
				T3_16BIT_RW &							// Compteur 16 bits
				T3_SOURCE_INT &							// Source interne (Quartz Fosc/4)
				T3_PS_1_1 &								// Prescaler 1/1
				T3_SYNC_EXT_OFF);						// Pas de synchronisation externe

	WriteTimer3(T_TIMER3);								// Charge le compteur du timer 3 avec la valeur de T_Timer3
	PIR2bits.TMR3IF=0;									// Timer3 Interrupt Flag = 0 Baisse le flag du timer 3
	IPR2bits.TMR3IP=0;									// Timer 3 dclench sur priorit basse
	RCONbits.IPEN=1;									// Autorise les niveau de priorit sur les interuptions
			
	
	// Autorise les interruptions	
	INTCONbits.PEIE=1;									// Autorise les interruptions de basse priorit
	INTCONbits.GIE=1;									// Autorise les interruptions de haute priorit
		
		
		
	
	ConsigneTourelle=VIT_TOURELLE;
	
	while (1)
	{	
		
		if (Donnee_Recue==0x53)
		{
			ConsigneTourelle=0;															// On arrte la tourelle
			while (Donnee_Recue==0x53);													// On attends une autre commande
			ConsigneTourelle=VIT_TOURELLE;														// On remet la tourelle en marche
			LargeurRaieMax=-1;															// Reinitialise la largeur de la meilleur raie connue												
			State=0; 																	// On retourne dans l'tat 0
		}
		
		switch (State)
		{
			// Etat initiale, on attends un front momtant (ou la fourche)
			
			case 0: 	if (Front_Descendant_Fourche()) { State=3; break; } 			// Si on dtecte un front sur la fourche, on remet passe dans l'tat 2
						if (Front_Reflexe()==2) 
						{	
							INTCONbits.GIE=0;											// On interdit les interruptions pour permettre le partage de la variable
							Debut=PositionTourelle;										// Mmorise la position du dbut du signal
							INTCONbits.GIE=1;											// On rautorise les interruptions
							State=1;													// Si on dtecte un front montant sur le capteur, on passe dans l'tat 1
						}
						break;
			
			
			// On attend que le signal repasse  0 (ou la fourche)
			
			case 1: 	if (Front_Descendant_Fourche()) { State=4; break; }				// Si on dtecte un front sur la fourche, on remet passe dans l'tat 2
						if (Front_Reflexe()==3) 
						{
							INTCONbits.PEIE=0;											// On interdit les interruptions pour permettre le partage de la variable
							Fin=PositionTourelle;										// Mmorise la position du fin du signal
							INTCONbits.PEIE=1;											// On rautorise les interruptions
							State=2;													// Si on dtecte un front descendant sur le capteur, on passe dans l'tat 2 pour le calcul
						}
						break;
			
			
			// On traite la trame fraichement reue
						
			case 2: 	if (Fin-Debut>LargeurRaieMax)									// Si c'est la meilleure raie connue
						{

							DebutMax=Debut;												// Mmorise le dbut de la trame							
							FinMax=Fin;													// Mmorise la fin de la trame
							LargeurRaieMax=Fin-Debut;									// Met  jour la meilleure largueur connue
						}			
						State=0;														// On retourne dans l'tat zro
						break;
					
			// On remet la position de la tourelle  0 (on a eu la fourche)					
						
			case 3: 	INTCONbits.PEIE=0;												// On interdit les interruptions pour permettre le partage de la variable
						PositionTourelle=0;												// On force la position de la tourelle  0
						INTCONbits.PEIE=1;												// On rautorise les interruptions
						
						//printf ("\n%d\t%d\n",DebutMax,FinMax);							
						if (LargeurRaieMax==-1)
						{
							DistaRobotAdverse=0;
							Led_Rouge(0);
						}
						else
						{
							Led_Rouge(1);
							AngleRobotAdverse=(int)((FinMax+DebutMax)*0.0413411458)%256;// Met en forme l'angle avec le robot adverse
// XM, 27/04
/*
							if (LargeurRaieMax>767) 
								DistaRobotAdverse=255;									// Bride la largeur max de la raie
							else
								DistaRobotAdverse=LargeurRaieMax/3;						// Met en forme l'information de distance
						}
*/
// remplac par
							if( LargeurRaieMax>=LMAX )
								DistaRobotAdverse = 1;
							else if( LargeurRaieMax<=LMIN )
								DistaRobotAdverse = 255;
							else
								// la distance, de 1  255 : 1 signifie au contact ; 255 signifie  peu prs 4m
								DistaRobotAdverse = 1 - 254.0/(LMAX-LMIN)*(LargeurRaieMax-LMAX);	
							
						}
// fin remplacement						

						#ifdef AFFICHAGE_SERIE
						printf ("%d %d\n", AngleRobotAdverse, DistaRobotAdverse );	
						#endif
						LargeurRaieMax=-1;												// Reinitialise la largeur de la meilleur raie connue												
						State=0; 														// On retourne dans l'tat 0
						break;														
		
		
			// On remet la position de la tourelle  0 (on a eu la fourche pendant la rception du signal
		
			case 4: 	INTCONbits.PEIE=0;												// On interdit les interruptions pour permettre le partage de la variable
						PositionTourelle=0;												// On force la position de la tourelle  0
						INTCONbits.PEIE=1;												// On rautorise les interruptions
						State=5; 														// On retourne dans l'tat 0
						break;		
					
			
			// La fourche vient de retomber  0		
						
			case 5:		if (Front_Descendant_Fourche()) { State=7; break; }				// Si on dtecte un front sur la fourche, Il y a un problme, on passe dans l'tat tampon en attendant que a redevienne normal
						if (Front_Reflexe()==3)
						{
							INTCONbits.PEIE=0;											// On interdit les interruptions pour permettre le partage de la variable
							Fin=PositionTourelle;										// Mmorise la position finale du signal
							INTCONbits.PEIE=1;											// On rautorise les interruptions
							State=6;													// Si on dtecte un front descendant sur le capteur, on passe dans l'tat 6
						}
						break;
				
			
			// Calcul + Remise  Zro + mise  jour avec un signal  cheval sur la fourche
				
			case 6: 	if (3072-Debut+Fin>LargeurRaieMax)								// Si c'est la meilleure raie connue
						{
							DebutMax=Debut;												// Mmorise le dbut de la trame
							FinMax=Fin+3072;											// Mmorise la fin de la trame, ajoute 3072  cause du passage par zro
							LargeurRaieMax=3072-Debut+Fin;								// Met  jour la meilleure largueur connue
						}	
						if (LargeurRaieMax==-1)
						{
							DistaRobotAdverse=0;
							Led_Rouge(0);
							//printf ("None ");
						}
						else
						{
							Led_Rouge(1);
							AngleRobotAdverse=(int)((FinMax+DebutMax)*0.0413411458)%256;
// XM, 27/04 :
/*							
							if (LargeurRaieMax>767) 
								DistaRobotAdverse=255;									// Bride la largeur max de la raie
							else
								DistaRobotAdverse=LargeurRaieMax/3;						// Met en forme l'information de distance
						}
*/
// remplac par
							if( LargeurRaieMax>=LMAX )
								DistaRobotAdverse = 1;
							else if( LargeurRaieMax<=LMIN )
								DistaRobotAdverse = 255;
							else
								// la distance, de 1  255 : 1 signifie au contact ; 255 signifie  peu prs 4m
								DistaRobotAdverse = 1 - 254.0/(LMAX-LMIN)*(LargeurRaieMax-LMAX);	
							
						}
// fin remplacement						
						#ifdef AFFICHAGE_SERIE
						printf ("%d %d\n",AngleRobotAdverse, DistaRobotAdverse );	
						#endif

												
						LargeurRaieMax=-1;												// Reinitialise la largeur de la meilleur raie connue								
						State=0;														// On retourne dans l'tat zro
						break;
		
			// On a fait un tour complet avec le signal  1, c'est pas normal !
		
			case 7: 	INTCONbits.PEIE=0;												// On interdit les interruptions pour permettre le partage de la variable
						PositionTourelle=0;												// On force la position de la tourelle  0
						INTCONbits.PEIE=1;												// On rautorise les interruptions
						State=8; 														// On retourne dans l'tat 0
						break;
				
			
			// Remise  zro avant de retourner dans l'tat initial	
											
			case 8: 	if (Front_Descendant_Fourche()) { State=7; break; }				// On a encore fait un tour complet, on retourne en 7
						if (Front_Reflexe()==3) 										// Si on a un front descendant on revient en 0
						{
							State=0;													// Le signal ets enfin tomb  0, on retourne en 0
							LargeurRaieMax=-1;											// Remise  zro, tout est plant !
						}
						break;
						
		} // fin switch
		
		


		
	} // fin while(1)

}






/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Interruptions                                // 
//                                                                         // 
/////////////////////////////////////////////////////////////////////////////



//----------------------------------------------------------------------------
// Vecteur d'interruptions de priorit haute (utilis pour l'IC)
#pragma code highVector=0x08						// Vecteur d'interruption haut
void atInterruptHigh(void)
{
	_asm goto Interrupt_High_Priority _endasm		// On saute  la fonction MyHighInterrupt
}
#pragma code 										// Retour  la zone de code




//----------------------------------------------------------------------------
// Vecteur d'interruptions de priorit basse (utilis pour le Timer)
#pragma code lowVector=0x18							// Vecteur d'interruption de priorit basse
void atInterruptLow(void)
{
	_asm goto Interrupt_Low_Priority _endasm		// On saute  la fonction MyHighInterrupt
}
#pragma code 										// Retour  la zone de code




//----------------------------------------------------------------------------
// Routine de service de l'interruption haute priorite
#pragma interrupt Interrupt_High_Priority
void Interrupt_High_Priority(void)
{
	int ret;
	
	if (PIR1bits.SSPIF) 									// C'est le MSSP qui a lev l'interruption, c'est pour l'IC
	{
		PIR1bits.SSPIF = 0; 								// On rautorise l'interruption		
		Led_Ambre(1);										// Led ambre pour marquer la requette IC
		Donnee_Recue=InterruptI2C(AngleRobotAdverse,DistaRobotAdverse);	// On appelle la routine I2C pour traiter la requete
		Led_Ambre(0);										// On etteinds la led
	}		
}





//----------------------------------------------------------------------------
// Routine de service de l'interruption haute priorite
#pragma interruptlow Interrupt_Low_Priority s
void Interrupt_Low_Priority(void)
{
	int 			Speed;											// Vitesse mesure au codeur
	float 			Commande=0;										// Commande applique au moteur
	static float 	CommandeOld=0;									// Commande du cycle prcdent
	float 			Error=0;										// Erreur en vitesse
	static float 	ErrorOld=0;										// Erreur du cycle prcdent
		
	if(PIR2bits.TMR3IF)												// Vrifie qu'il s'agit bien de l'interruption du Timer 3
	{									
		WriteTimer3(T_TIMER3);										// Recharge le compteur du Timer 3		
		PIR2bits.TMR3IF=0;											// Rautotise l'interruption
		Speed=Codeur_Vitesse();										// Lecture de la vitesse du moteur
		PositionTourelle=PositionTourelle+Speed;					// Met  jour la position de la tourelle
		if (PositionTourelle>3072) PositionTourelle=3072;			// Evite les dbordements
		while (PositionTourelle<0)									// Bride entre 0 et 3072
			PositionTourelle+=3072;
		Error=(float)(ConsigneTourelle-Speed);						// Calcul l'erreur en vitesse
		Commande=K*Alpha*Error-K*ErrorOld+CommandeOld;				// Calcule le PI
		CommandeOld=Commande;										// Mmorise la commande
		ErrorOld=Error;												// Mmorise l'erreur
		if (Commande>COMMANDE_MAX) Commande=COMMANDE_MAX;			// Bride la commande
		if (Commande<-COMMANDE_MAX) Commande=-COMMANDE_MAX;
		CommandeMoteur ((int)Commande);								// Envoie la commande sur le moteur		
	}	
}
	